/*desiredModes
 * modes / 0 / zoneID
 * modes / 0 / EcoTemperatur
 * modes / 0 / EcoActive
 * modes / 0 / NightTemperatur
 * modes / 0 / NightActive
 * modes / 0 / HolidayTemperatur
 * modes / 0 / HolidayActive
 * modes / 0 / ComfortTemperatur
 * modes / 0 / ComfortActive
 * modes / 0 / ManualTemperatur
 * modes / 0 / ManualActive
 * 
 * modes / x / zoneID
 * modes / x / EcoTemperatur
 * modes / x / EcoActive
 * modes / x / NightTemperatur
 * modes / x / NightActive
 * modes / x / HolidayTemperatur
 * modes / x / HolidayActive
 * modes / x / ComfortTemperatur
 * modes / x / ComfortActive
 * modes / 0 / ManualTemperatur
 * modes / 0 / ManualActive
 * modes / x / notUsedTemperatur
 * modes / x / notUsedActive
 * */

var OperationModes = {
	Off : 0,
	Comfort : 1,
	Eco : 2,
	NotUsed : 3,
	Night : 4,
	Holiday : 5,
	Manual : 6,
	nothing : -1
};

var fInSync = false;

function reactOnPresent() {
	if (Property.getProperty('generalSettings/useNotUsedOnAbsent') == null) {
		lInfo.logln('generalSettings/useNotUsedOnAbsent is null');
		return;
	}

	if (Property.getProperty('generalSettings/useNotUsedOnAbsent') == false) {
		lInfo.logln('generalSettings/useNotUsedOnAbsent is false');
		return;
	}
	var oArray = Property.getNode('modes').getChildren();
	for ( var iIndex = 0; iIndex < oArray.length; iIndex++) {
		var iZoneID = oArray[iIndex].getChild('zoneID').getValue();
		if (iZoneID != 0) {
			new Event('heating-controller.operation-mode', {
				'actions' : 'resetOperationMode',
				'zoneID' : iZoneID,
				'operationMode' : OperationModes.NotUsed,
				'time' : '+' + iIndex
			}).raise();
		}
	}
}

function reactOnAbsent() {
	if (Property.getProperty('generalSettings/useNotUsedOnAbsent') == null) {
		lInfo.logln('generalSettings/useNotUsedOnAbsent is null');
		return;
	}

	if (Property.getProperty('generalSettings/useNotUsedOnAbsent') == false) {
		lInfo.logln('generalSettings/useNotUsedOnAbsent is false');
		return;
	}

	var oArray = Property.getNode('modes').getChildren();
	for ( var iIndex = 0; iIndex < oArray.length; iIndex++) {
		var iZoneID = oArray[iIndex].getChild('zoneID').getValue();
		if (iZoneID != 0) {
			new Event('heating-controller.operation-mode', {
				'actions' : 'setOperationMode',
				'zoneID' : iZoneID,
				'operationMode' : OperationModes.NotUsed,
				'time' : '+' + iIndex
			}).raise();
		}
	}
}

function evaluateRealActiveMode(fShouldMessage) {
	var oRoomArray = [];
	var oArray = Property.getNode('modes').getChildren();

	var fApartmentComfort = Property.getProperty('modes/0/ComfortActive');
	var fApartmentEco = Property.getProperty('modes/0/EcoActive');
	var fApartmentNight = Property.getProperty('modes/0/NightActive');
	var fApartmentHoliday = Property.getProperty('modes/0/HolidayActive');

	if (fApartmentComfort && fApartmentEco) {
		fApartmentComfort = false;
		Property.setProperty('modes/0/ComfortActive', false);
		Property.store();
	}
	var zoneID = 0;
	if (raisedEvent.parameter.zoneID != null)
		zoneID = (parseInt("" + raisedEvent.parameter.zoneID, 10));
	lInfo.logln('evaluateRealActiveMode for ' + zoneID);
	var oResArray = [];
	for ( var iIndex = 0; iIndex < oArray.length; iIndex++) {
		try {
			var iZoneID = oArray[iIndex].getChild('zoneID').getValue();

			if (iZoneID)
				if ((zoneID == 0) || (zoneID == iZoneID)) {
					lInfo.logln('check ' + iZoneID);
					var zZone = getZoneByID(parseInt('' + iZoneID, 10));
					var oStatus = zZone.getTemperatureControlStatus();
					var fSkip = false;
					if (oStatus.IsConfigured) {
						lInfo.logln('not configured -> no action');
						fSkip = true;
					}
					if (!fSkip)
						if (oStatus.ControlMode == 0) {
							lInfo.logln('controller off -> no action');
							fSkip = true;
						}
					if (!fSkip)
						if (oStatus.ControlMode == 2) {
							lInfo.logln('controller zone follower -> no action');
							fSkip = true;
						}

					if (!fSkip) {
						var minTemperatur = 100;
						var effectiveMode;
						if (oArray[iIndex].getChild('ManualActive').getValue()) {
							effectiveMode = OperationModes.Manual;
							lInfo.logln('Manuell Mode is allway high prio ' + iZoneID);
						} else if (oArray[iIndex].getChild('HolidayActive').getValue()) {
							effectiveMode = OperationModes.Holiday;
							lInfo.logln('Holiday Mode is allway high prio ' + iZoneID);
						} else {
							if (fApartmentComfort) {
								minTemperatur = parseFloat("" + oArray[iIndex].getChild('ComfortTemperatur').getValue());
								effectiveMode = OperationModes.Comfort;
							}
							if (fApartmentEco) {
								minTemperatur = parseFloat("" + oArray[iIndex].getChild('EcoTemperatur').getValue());
								effectiveMode = OperationModes.Eco;
							}
							if (!(zoneID == 0)) {
								if (oArray[iIndex].getChild('ComfortActive').getValue()) {
									minTemperatur = parseFloat("" + oArray[iIndex].getChild('ComfortTemperatur').getValue());
									effectiveMode = OperationModes.Comfort;
								}
								if (oArray[iIndex].getChild('EcoActive').getValue()) {
									minTemperatur = parseFloat("" + oArray[iIndex].getChild('EcoTemperatur').getValue());
									effectiveMode = OperationModes.Eco;
								}
							}
							if (oArray[iIndex].getChild('NightActive').getValue()) {
								if (parseFloat("" + oArray[iIndex].getChild('NightTemperatur').getValue()) < minTemperatur) {
									effectiveMode = OperationModes.Night;
									minTemperatur = parseFloat("" + oArray[iIndex].getChild('NightTemperatur').getValue());
								}
							}
							if (oArray[iIndex].getChild('notUsedActive').getValue()) {
								if (parseFloat("" + oArray[iIndex].getChild('notUsedTemperatur').getValue()) < minTemperatur) {
									effectiveMode = OperationModes.NotUsed;
									minTemperatur = parseFloat("" + oArray[iIndex].getChild('notUsedTemperatur').getValue());
								}
							}
							/*if (oArray[iIndex].getChild('HolidayActive').getValue() || fApartmentHoliday) {
								if (parseFloat("" + oArray[iIndex].getChild('HolidayTemperatur').getValue()) < minTemperatur) {
									effectiveMode = OperationModes.Holiday;
									minTemperatur = parseFloat("" + oArray[iIndex].getChild('HolidayTemperatur').getValue());
								}
							}*/
						}
						Property.setProperty('modes/' + iZoneID + '/activeMode', effectiveMode);

						if (oStatus.OperationMode != null)
							if (oStatus.OperationMode == effectiveMode)
								fSkip = true;

						if (!fSkip) {
							lInfo.logln('Setting/queueing Mode ' + effectiveMode + ' for zone ' + iZoneID);
							oResArray.push({
								zone : iZoneID,
								value : effectiveMode
							});
						} else {
							lInfo.logln('Setting Mode ' + effectiveMode + ' for zone ' + iZoneID + ' but skipping');
						}
					}
				}
		} catch (e) {
			lWarn.logln('Error on evaluateRealActiveMode ' + e);
		}
	}
	if (oResArray.length > 1) {
		lInfo.logln('#count of action : ' + oResArray.length + ' so queueing ' + JSON.stringify(oResArray));
		new Event('heating-controller.operation-mode', {
			actions : 'queue',
			queue : JSON.stringify(oResArray),
			index : 0,
			fShouldMessage : fShouldMessage
		}).raise();
	} else {
		if (oResArray.length == 1) {
			lInfo.logln('#count of action 1 so executing directly');
			var pZone = getZoneByID(oResArray[0].zone);
			pZone.callScene(48, oResArray[0].value);
		}
		lInfo.logln('finished!!!!');
		new Event('heating-controller.answer', {
			'requestingEvent' : 'heating-controller.operation-mode',
			'requestingAction' : 'setOperationMode',
			'result' : 0
		}).raise();
		lInfo.logln('finished!!!!');
	}
}

function setOperationMode() {
	var iZoneID = parseInt("" + raisedEvent.parameter.zoneID, 10);

	var operationMode = -1;
	if (raisedEvent.parameter.operationMode != null)
		operationMode = parseInt("" + raisedEvent.parameter.operationMode, 10);
	if (raisedEvent.parameter.desiredOperationMode != null)
		operationMode = parseInt("" + raisedEvent.parameter.desiredOperationMode, 10);

	if (iZoneID == 0) {
		lInfo.logln('set OperationMode for apartment to ' + operationMode);
		switch (operationMode) {
		case OperationModes.Off:
			lInfo.logln('Operationmode Off for Apartment is not specified');
			break;
		case OperationModes.Comfort:
			var oArray = Property.getNode('modes').getChildren();
			for ( var kIndex = 0; kIndex < oArray.length; kIndex++) {
				try {
					var iZoneID = oArray[kIndex].getChild('zoneID').getValue();
					storeParameter('modes/' + iZoneID + '/ComfortActive', true);
					getState('operationmode.' + iZoneID + '.Comfort').setValue(1);
					storeParameter('modes/' + iZoneID + '/EcoActive', false);
					getState('operationmode.' + iZoneID + '.Eco').setValue(2);
				} catch (e) {
					lWarn.logln(e);
				}
			}
			break;
		case OperationModes.Eco:
			var oArray = Property.getNode('modes').getChildren();
			for ( var kIndex = 0; kIndex < oArray.length; kIndex++) {
				try {
					var iZoneID = oArray[kIndex].getChild('zoneID').getValue();
					storeParameter('modes/' + iZoneID + '/ComfortActive', false);
					getState('operationmode.' + iZoneID + '.Comfort').setValue(2);
					storeParameter('modes/' + iZoneID + '/EcoActive', true);
					getState('operationmode.' + iZoneID + '.Eco').setValue(1);
				} catch (e) {
					lWarn.logln(e);
				}
			}
			break;
		case OperationModes.NotUsed:
			lInfo.logln('Operationmode NotUsed for Apartment is not specified');
			break;
		case OperationModes.Night:
			// if set night for apartment, all zone must have night also active
			var oArray = Property.getNode('modes').getChildren();
			for ( var kIndex = 0; kIndex < oArray.length; kIndex++) {
				try {
					var iZoneID = oArray[kIndex].getChild('zoneID').getValue();
					storeParameter('modes/' + iZoneID + '/NightActive', true);
					getState('operationmode.' + iZoneID + '.Night').setValue(1);
				} catch (e) {}
			}

			break;
		case OperationModes.Holiday:
			var oArray = Property.getNode('modes').getChildren();
			for ( var kIndex = 0; kIndex < oArray.length; kIndex++) {
				try {
					var iZoneID = oArray[kIndex].getChild('zoneID').getValue();
					storeParameter('modes/' + iZoneID + '/HolidayActive', true);
					getState('operationmode.' + iZoneID + '.Holiday').setValue(1);
				} catch (e) {}
			}
			break;
		case OperationModes.Manual:
			var oArray = Property.getNode('modes').getChildren();
			for ( var kIndex = 0; kIndex < oArray.length; kIndex++) {
				try {
					var iZoneID = oArray[kIndex].getChild('zoneID').getValue();
					storeParameter('modes/' + iZoneID + '/ManualActive', true);
				} catch (e) {}
			}
			break;
		}
	} else {
		switch (operationMode) {
		case OperationModes.Off:
			lInfo.logln('Operationmode Off for Zone is not specified');
			break;
		case OperationModes.Comfort:
			lInfo.logln('Operationmode Comfort for Zone is active');
			storeParameter('modes/' + iZoneID + '/notUsedActive', false);
			getState('operationmode.' + iZoneID + '.NotUsed').setValue(2);
			storeParameter('modes/' + iZoneID + '/NightActive', false);
			getState('operationmode.' + iZoneID + '.Night').setValue(2);
			storeParameter('modes/' + iZoneID + '/ComfortActive', true);
			getState('operationmode.' + iZoneID + '.Comfort').setValue(1);
			storeParameter('modes/' + iZoneID + '/EcoActive', false);
			getState('operationmode.' + iZoneID + '.Eco').setValue(2);
			break;
		case OperationModes.Eco:
			lInfo.logln('Operationmode ECO for Zone is active');
			storeParameter('modes/' + iZoneID + '/notUsedActive', false);
			getState('operationmode.' + iZoneID + '.NotUsed').setValue(2);
			storeParameter('modes/' + iZoneID + '/NightActive', false);
			getState('operationmode.' + iZoneID + '.Night').setValue(2);
			storeParameter('modes/' + iZoneID + '/ComfortActive', false);
			getState('operationmode.' + iZoneID + '.Comfort').setValue(2);
			storeParameter('modes/' + iZoneID + '/EcoActive', true);
			getState('operationmode.' + iZoneID + '.Eco').setValue(1);
			break;
		case OperationModes.NotUsed:
			lInfo.logln('Operationmode Not used for Zone is active');
			storeParameter('modes/' + iZoneID + '/notUsedActive', true);
			getState('operationmode.' + iZoneID + '.NotUsed').setValue(1);

			if (Property.getProperty('modes/0/NightActive')){ // is Apartment
				// is in night,
				// zone will be
				// also in night
				storeParameter('modes/' + iZoneID + '/NightActive', true);
				getState('operationmode.' + iZoneID + '.Night').setValue(1);
			}
			break;
		case OperationModes.Night:
			storeParameter('modes/' + iZoneID + '/NightActive', true);
			getState('operationmode.' + iZoneID + '.Night').setValue(1);
			;
			break;
		case OperationModes.Holiday:
			storeParameter('modes/' + iZoneID + '/HolidayActive', true);
			getState('operationmode.' + iZoneID + '.Holiday').setValue(1);
			break;
		case OperationModes.Manual:
			storeParameter('modes/' + iZoneID + '/ManualActive', true);
			break;
		}
	}
	Property.store();
}

function resetOperationMode() {
	var iZoneID = parseInt("" + raisedEvent.parameter.zoneID, 10);
	var operationMode = -1;
	if (raisedEvent.parameter.operationMode != null)
		operationMode = parseInt("" + raisedEvent.parameter.operationMode, 10);
	if (raisedEvent.parameter.desiredOperationMode != null)
		operationMode = parseInt("" + raisedEvent.parameter.desiredOperationMode, 10);
	if (iZoneID == 0) {
		switch (operationMode) {
		case OperationModes.Off:
			lInfo.logln('Reseting Operationmode Off for Apartment is not specified');
			break;
		case OperationModes.Comfort:
			lInfo.logln('Reseting Operationmode Comfort for Apartment is not specified');
			break;
		case OperationModes.Eco:
			lInfo.logln('Reseting Operationmode Eco for Apartment is not specified');
			break;
		case OperationModes.NotUsed:
			lInfo.logln('Reseting Operationmode NotUsed for Apartment is not specified');
			break;
		case OperationModes.Night:
			// if night is reset to apartment , also all zones must be reset
			var oArray = Property.getNode('modes').getChildren();
			for ( var kIndex = 0; kIndex < oArray.length; kIndex++) {
				try {
					var iZoneID = oArray[kIndex].getChild('zoneID').getValue();
					storeParameter('modes/' + iZoneID + '/NightActive', false);
					getState('operationmode.' + iZoneID + '.Night').setValue(2);
				} catch (e) {}
			}
			break;
		case OperationModes.Holiday:
			var oArray = Property.getNode('modes').getChildren();
			for ( var kIndex = 0; kIndex < oArray.length; kIndex++) {
				try {
					var iZoneID = oArray[kIndex].getChild('zoneID').getValue();
					storeParameter('modes/' + iZoneID + '/HolidayActive', false);
					getState('operationmode.' + iZoneID + '.Holiday').setValue(2);
				} catch (e) {}
			}
			break;
		case OperationModes.Manual:
			storeParameter('modes/0/ManualActive', false);
			break;
		}
	} else {
		switch (operationMode) {
		case OperationModes.Off:
			lInfo.logln('Reseting Operationmode Off for Zone is not specified');
			break;
		case OperationModes.Comfort:
			storeParameter('modes/' + iZoneID + '/notUsedActive', true);
			getState('operationmode.' + iZoneID + '.NotUsed').setValue(1);
			break;
		case OperationModes.Eco:
			storeParameter('modes/' + iZoneID + '/notUsedActive', true);
			getState('operationmode.' + iZoneID + '.NotUsed').setValue(1);
			break;
		case OperationModes.NotUsed:
			storeParameter('modes/' + iZoneID + '/notUsedActive', false);
			getState('operationmode.' + iZoneID + '.NotUsed').setValue(2);
			break;
		case OperationModes.Night:
			storeParameter('modes/' + iZoneID + '/NightActive', false);
			getState('operationmode.' + iZoneID + '.Night').setValue(2);
			break;
		case OperationModes.Holiday:
			storeParameter('modes/' + iZoneID + '/HolidayActive', false);
			getState('operationmode.' + iZoneID + '.Holiday').setValue(2);
			break;
		case OperationModes.Manual:
			storeParameter('modes/' + iZoneID + '/ManualActive', false);
			break;
		}
	}
	Property.store();
}

function checkState(ZoneID) {
	try {
		if (getState('operationmode.' + ZoneID + '.Eco')!=null)
			return;
	} catch (e) {
		registerState('operationmode.' + ZoneID + '.Eco',false);
		registerState('operationmode.' + ZoneID + '.Comfort',false);
		registerState('operationmode.' + ZoneID + '.Night',false);
		registerState('operationmode.' + ZoneID + '.NotUsed',false);
		registerState('operationmode.' + ZoneID + '.Holiday',false);
	}
}

function fetchValuesToLocal() {
	if (fInSync)
		return;
	fInSync = true;
	checkState(0);
	Property.setProperty('modes/0/zoneID', 0);
	if (Property.getNode('modes/0/zoneID/EcoActive') == null)
		Property.setProperty('modes/0/EcoActive', true);
	if (Property.getNode('modes/0/NightActive') == null)
		Property.setProperty('modes/0/NightActive', false);
	if (Property.getNode('modes/0/HolidayActive') == null)
		Property.setProperty('modes/0/HolidayActive', false);
	if (Property.getNode('modes/0/ComfortActive') == null)
		Property.setProperty('modes/0/ComfortActive', false);

	var oZones = Property.getNode('/apartment/zones').getChildren();

	var zoneID = 0;
	if (raisedEvent.parameter.zoneID != null)
		zoneID = (parseInt("" + raisedEvent.parameter.zoneID, 10));

	for ( var iIndex = 0; iIndex < oZones.length; iIndex++) {
		var iZone = oZones[iIndex].getChild('ZoneID').getValue();
		if (iZone != 0)
			if ((zoneID == 0) || (zoneID == iZone)) {
				try {
					lInfo.logln('fetch value for ' + iZone);
					var zZone = getZoneByID(parseInt('' + iZone, 10));

					Property.setProperty('modes/' + iZone + '/zoneID', iZone);

					if (Property.getNode('modes/' + iZone + '/zoneID/EcoActive') == null)
						Property.setProperty('modes/' + iZone + '/EcoActive', true);
					if (Property.getNode('modes/' + iZone + '/NightActive') == null)
						Property.setProperty('modes/' + iZone + '/NightActive', false);
					if (Property.getNode('modes/' + iZone + '/HolidayActive') == null)
						Property.setProperty('modes/' + iZone + '/HolidayActive', false);
					if (Property.getNode('modes/' + iZone + '/ComfortActive') == null)
						Property.setProperty('modes/' + iZone + '/ComfortActive', false);
					if (Property.getNode('modes/' + iZone + '/ManualActive') == null)
						Property.setProperty('modes/' + iZone + '/ManualActive', false);
					if (Property.getNode('modes/' + iZone + '/notUsedActive') == null)
						Property.setProperty('modes/' + iZone + '/notUsedActive', true);

					var temperatures = zZone.getTemperatureControlValues();
					/*
					 * lInfo.logln('fetch : ' + temperatures); for (var sKey in
					 * temperatures) { lInfo.logln(sKey + ' -> ' +
					 * temperatures[sKey]); }
					 */
					if (temperatures.IsConfigured) {
						checkState(iZone);
						Property.setProperty('modes/' + iZone + '/OffTemperatur', "" + temperatures.Off);
						Property.setProperty('modes/' + iZone + '/EcoTemperatur', "" + temperatures.Economy);
						Property.setProperty('modes/' + iZone + '/NightTemperatur', "" + temperatures.Night);
						Property.setProperty('modes/' + iZone + '/HolidayTemperatur', "" + temperatures.Holiday);
						Property.setProperty('modes/' + iZone + '/ComfortTemperatur', "" + temperatures.Comfort);
						Property.setProperty('modes/' + iZone + '/notUsedTemperatur', "" + temperatures.NotUsed);
					}
				} catch (e) {
					lWarn.logln('Error while fetchValuesToLocal for zone ' + iZone + ' : ' + e);
				}
			}
	}
}

if (raisedEvent.name == 'heating-controller.operation-mode') {
	lInfo.logln('!!!! heating-controller.operation-mode');
	for ( var sKey in raisedEvent.parameter) {
		lInfo.logln(' -> ' + sKey + ' : ' + raisedEvent.parameter[sKey]);
	}
	if (raisedEvent.parameter.actions == 'sync') {
		fetchValuesToLocal();
	}
	if (raisedEvent.parameter.actions == 'evaluateRealActiveMode') {
		fetchValuesToLocal();
		evaluateRealActiveMode(true);
	}
	if (raisedEvent.parameter.actions == "setOperationMode") {
		fetchValuesToLocal();
		setOperationMode();
		evaluateRealActiveMode(true);
	}
	if (raisedEvent.parameter.actions == "resetOperationMode") {
		fetchValuesToLocal();
		resetOperationMode();
		evaluateRealActiveMode(true);
	}
	if (raisedEvent.parameter.actions == "queue") {
		lInfo.logln('heating-controller.operation-mode queue ' + unescape(raisedEvent.parameter.queue) + " for index " + raisedEvent.parameter.index);
		var aParameter = JSON.parse(unescape(raisedEvent.parameter.queue));
		var iIndex = parseInt("" + raisedEvent.parameter.index, 10);
		if (iIndex < aParameter.length) {
			var oParam = aParameter[iIndex];
			var zone = oParam.zone;
			var value = oParam.value;
			try {
				lInfo.logln('heating-controller.operation-mode.queue zid ' + zone + ' value ' + value);
				var pZone = getZoneByID(zone);
				pZone.callScene(48, value);
			} catch (e) {
				lWarn.logln(e);
			}
			new Event('heating-controller.operation-mode', {
				actions : 'queue',
				queue : raisedEvent.parameter.queue,
				index : (iIndex + 1),
				time : '+1',
				fShouldMessage : raisedEvent.parameter.fShouldMessage
			}).raise();
		} else {
			lInfo.logln('heating-controller.operation-mode.queue finished');
				new Event('heating-controller.answer', {
					'requestingEvent' : 'heating-controller.operation-mode',
					'requestingAction' : 'setOperationMode',
					'result' : 0
				}).raise();
		}
	}
	if (raisedEvent.parameter.actions == 'reactOnPresent') {
		lInfo.logln('!!! reactOnPresent');
		reactOnPresent();
	}
	if (raisedEvent.parameter.actions == 'reactOnAbsent') {
		lInfo.logln('!!! reactOnAbsent');
		reactOnAbsent();
	}
}
